'*************************************
'*   i8080_emu v0.0                  *
'*   An emulation of the Intel i8080 *
'*   (C) 2009 Michael Rychlik        *
'*************************************
'
'v0.0 - 27 Jan 2009 - Original version

'This file is a minimal example of how to set up and run the Intel i8080_emu.


CON

  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000

  ram_size = (1024 * 24)
  rom_start = $FF00

OBJ

  cpu_0   : "i8080_emu_0"    'i8080 emulator part 0
  cpu_1   : "i8080_emu_1"    'i8080 emulator part 1
  cpu_2   : "i8080_emu_2"    'i8080 emulator part 2
  cpu_3   : "i8080_emu_3"    'i8080 emulator part 3
  
  kb      : "PC_Keyboard"    'You can also use Keyboard object for PS2-Keyboard                          
  TV      : "PC_Text"        'You can also use TV_Text or VGA_Text objects

VAR     

'Parameter list for CPU objects, do not change order.
long reg_base       'Address (in HUB)of 8080 register file
long ram_base       'Address (in HUB) of the 8080 RAM area
long ram_len        'Size of 8080 RAM area, assummed to start at zero in 8080 address space.
long rom_base       'Address (in HUB) of the 8080 ROM area
long rom_addr       'Address in 8080 RAM area of the start of ROM.
long io_base        'Address (in prop RAM) of the I/O contol interface
long cpu_control_ptr'Quick hack at single step control

word err            'Error code return value
long cpu_control

DAT

'The 8080 Registers.
af_reg        word      0
bc_reg        word      0 
de_reg        word      0
hl_reg        word      0
sp_reg        word      0
pc_reg        word      0

'The 8080 will start executing instructions from here
ram_8080
              byte      %00000100   'INR B   ;by CPU 0
              byte      %01101000   'MOV L,B ;by CPU 1
              byte      %10000000   'ADD B   ;by CPU 2
              byte      %11000011   'JMP 0   ;by CPU 3
              byte      %00000000
              byte      %00000000
ram_8080_end

'ROM is assumed to be top of the 8080 memory map
rom_8080
              byte      0[256]
rom_8080_end

PUB start

  ram_base := @ram_8080
  ram_len := ram_size
  rom_base := @rom_8080
  rom_addr := rom_start
  'ios_ctl_addr := @ctl_if
  cpu_control_ptr := @cpu_control
  
  'Start the terminal
  TV.start(12)
  TV.str(string("Propeller i8080_emu v0.0",13))
  
  'Start the keyboard
  err := \kb.start(26, 27)
  TV.str (string("Starting keyboard ... "))
  if err == 0
    TV.str (string("FAIL",13))
  else
    TV.str (string("OK",13))

  'Initialize i8080 registers
  af_reg := $0000
  bc_reg := $0000
  de_reg := $0000
  hl_reg := $0000
  sp_reg := $0000
  pc_reg := $0000

  print_regs

  reg_base := @af_reg

  TV.str(string("RAM base = "))
  TV.hex(ram_base, 4)
  TV.out(13)

  TV.str(string("ROM base = "))
  TV.hex(rom_base, 4)
  TV.out(13)


  TV.str(string("Starting CPU COG 0..."))
    err := \cpu_0.start(@reg_base)
  if err == 0
    TV.str (string("FAIL",13))
  else
    TV.str (string("OK",13))


  TV.str(string("Starting CPU COG 1..."))
    err := \cpu_1.start(@reg_base)
  if err == 0
    TV.str (string("FAIL",13))
  else
    TV.str (string("OK",13))

  TV.str(string("Starting CPU COG 2..."))
    err := \cpu_2.start(@reg_base)
  if err == 0
    TV.str (string("FAIL",13))
  else
    TV.str (string("OK",13))

  TV.str(string("Starting CPU COG 3..."))
    err := \cpu_3.start(@reg_base)
  if err == 0
    TV.str (string("FAIL",13))
  else
    TV.str (string("OK",13))


  repeat
    cpu_control := $1
    repeat while cpu_control <> 0
    print_regs
    waitcnt(40_000_000 + cnt)



PRI print_regs
  TV.str(string("PC="))
  TV.hex(pc_reg, 4)
  TV.out(" ")

  TV.str(string("AF="))
  TV.hex(af_reg, 4)
  TV.out(" ")
      
  TV.str(string("BC="))
  TV.hex(bc_reg, 4)
  TV.out(" ")
      
  TV.str(string("DE="))
  TV.hex(de_reg, 4)
  TV.out(" ")

  TV.str(string("HL="))
  TV.hex(hl_reg, 4)
  TV.out(" ")
      
  TV.str(string("SP="))
  TV.hex(sp_reg, 4)
  TV.out(13)

    