'*************************************      
'*  i8080_emu v0.0                   *
'*  An emulation of the Intel i8080  *
'*  (C) 2009 Michael Rychlik         *
'*************************************
'
'v0.0 - 27 Jan 2009 - Original version

CON

  '8080 Program Status Word bits
  sign_bit   = %10000000
  zero_bit   = %01000000
  aux_bit    = %00010000
  parity_bit = %00000100
  carry_bit  = %00000001

VAR
  long  cog

PUB start(cpu_params) : okay

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
''   cpu_par_params is a pointer to CPU parameter list

  object_base := @@0                            'Set up the base address of this object
  
  stop
  okay := cog := cognew(@enter, cpu_params) + 1 'Start emulator in a new COG

PUB stop

'' Stop CPU simulator - frees a cog

  if cog
    cogstop(cog~ - 1)


DAT
                        org     0

enter                   mov     cpu_params_ptr, par
                        rdlong  reg_ptr, cpu_params_ptr    'First item in CPU params list is 
                        mov     psw_reg, reg_ptr           'pointer to register file.
                        add     reg_ptr, #1
                        mov     a_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     c_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     b_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     e_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     d_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     l_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     h_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     sp_reg, reg_ptr
                        add     reg_ptr, #2
                        mov     pc_reg, reg_ptr
                        add     cpu_params_ptr, #4         'Move parAms pointer up to ram_base
                        rdlong  ram_base, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to ram_len
                        rdlong  ram_size, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to rom base
                        rdlong  rom_base, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to rom address
                        rdlong  rom_address, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to I/O interface add
                                                           'FIXME not implemented yet
                        add     cpu_params_ptr, #4         'Move params pointer up CPU control address
                        rdlong  cpu_control, cpu_params_ptr

'---------------------------------------------------------------------------------------------------------
' Instruction fetch and execute loop. Common to all CPU emulator COGS

                        add     dispatch_tab_addr, object_base 'Fixup this object offset to HUB real address
                        
                        rdword  pc, pc_reg                 'Initialize COG program counter from global 
                        jmp     #fetch                     'program counter and fetch first instruction

done
                        wrword  pc, pc_reg                 'Instruction executed so up date global (HUB)
                                                           'Program Counter and fetch next instruction

                        mov     data_16, #0                '\ DEBUG CODE
                        wrword  data_16, cpu_control       '/
                        
                                                           'Instruction fetch loop
fetch                   call    #read_immediate_byte       'Get the instruction into 'data_8'

halt                    rdword  data_16, cpu_control       '\
                        cmp     data_16, #0 wz             ' | DEBUG CODE 
              if_z      jmp     #halt                      '/

                        mov     scratch, data_8            'Dispatch table look up
                        shl     scratch, #1                'Mul by 2 to get a WORD offset into the table
                        add     scratch, dispatch_tab_addr 'Index into table
                        rdword  scratch, scratch           'Read the instruction handlers address
                        jmp     scratch                    'Jump to instruction handler

skip                    rdword  pc_next, pc_reg            'Wait until global (HUB) Program Counter
                        cmp     pc_next, pc                'chanages. I.E. another COG exexuted this op.
              if_z      jmp     #skip
                        mov     pc, pc_next                'PC changed so lets go
                        jmp     #fetch

scratch                 long    0
dispatch_tab_addr       long    @dispatch_table
object_base             long    0-0                        'HUB address of this object 
cpu_params_ptr          long    0
reg_ptr                 long    0
pc                      long    0                          'Local COG program conter
pc_next                 long    0

'---------------------------------------------------------------------------------------------------------
cpu_control             long    0
'---------------------------------------------------------------------------------------------------------
'Functions common (duplicated in) all CPU emulator COGs


'Read a BYTE from location given by "address" of 8080 RAM into byte data_8
read_memory_byte        cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_b      jmp     #:read_ram
                        call    #read_rom
                        jmp     #read_memory_byte_ret
:read_ram               mov     hub_pointer, ram_base      'Point to the RAM
                        add     hub_pointer, address       'Add address offset
                        rdbyte  data_8, hub_pointer        'Read the byte from there
read_memory_byte_ret    ret


read_rom                cmp     address, rom_start wc,wz   'Is the given address within the ROM range ?
              if_b      jmp     #read_rom_ret              'address below rom_base => return with "data" undefined
                        mov     rom_address, address
                        sub     rom_address, rom_start     'address becomes offset into ROM from start of ROM                  
                        mov     hub_pointer, rom_base      'Point to the ROM
                        add     hub_pointer, rom_address   'Add address offset                     
                        rdbyte  data_8, hub_pointer        'Read the byte from there
read_rom_ret            ret                                'Return


'Write the BYTE data_8 to location given by "address"
write_memory_byte       cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_b      mov     hub_pointer, ram_base      'Point to the RAM
              if_b      add     hub_pointer, address       'Add address offset
              if_b      wrbyte  data_8, hub_pointer        'Write the byte there
write_memory_byte_ret   ret


'Read a WORD from location given by "address" of 8080 RAM into byte data_16
read_memory_word        call    #read_memory_byte          'Read the low byte
                        mov     data_16, data_8
                        ror     data_16, #8
                        add     address, #1
                        call    #read_memory_byte          'Read the high byte
                        or      data_16, data_8
                        rol     data_16, #8
read_memory_word_ret    ret

'Write the WORD data_16 to location given by "address"
write_memory_word       mov     data_8, data_16
                        call    #write_memory_byte         'Write the LOW byte
                        shr     data_8, #8
                        add     address, #1
                        call    #write_memory_byte
write_memory_word_ret   ret

'The following configuration "constants" set from COG PARameters.
ram_base                long    0-0                        'HUB address of the 8080 RAM area
ram_size                long    0-0                        'Length in bytes of available 8080 RAM
rom_base                long    0-0                        'HUB address of the BOOT ROM image            
rom_start               long    0-0                        'ROM starts at this address in 8080 space
rom_address             long    0-0

'8080 RAM read/write vars.
hub_pointer             long    0                          'Pointer into HUB RAM
address                 long    0                          'Address param for read/write funcs
data_8                  long    0                          'Data bytes read into here
data_16                 long    0                          'Data words read into here



'Read a BYTE from location given by "pc" of 8080 RAM into byte data_8 and increment pc
read_immediate_byte     mov     address, pc
                        call    #read_memory_byte
                        add     pc, #1
read_immediate_byte_ret ret

'Read a WORD from location given by "pc" of 8080 RAM into byte data_8 and pc = pc + 2
read_immediate_word     mov     address, pc
                        call    #read_memory_word
                        add     pc, #2
read_immediate_word_ret ret



'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Functions implementing the Arithmetic Logic Unit incrementer/decrementer
alu_increment           mov     aux, alu                   'Keep a copy for AUX carry determination
                        add     alu, #1                    'Do the increment
                        call    #alu_inc_dec_flags         'Set the flags 
alu_increment_ret       ret

alu_decrement           mov     aux, alu                   'Keep a copy for AUX carry determination
                        sub     alu, #1                    'Do the decrement
                        call    #alu_inc_dec_flags         'Set the flags 
alu_decrement_ret       ret

alu_inc_dec_flags       rdbyte  flags, psw_reg
                        xor     aux, alu                   'AUX carry ?
                        test    aux, #aux_bit wz
                        muxnz   flags, #aux_bit                        
                        and     alu, #$FF wz, wc
                        muxz    flags, #zero_bit
                        muxnc   flags, #parity_bit
                        test    alu, #128 wz
                        muxnz   flags, #sign_bit
                        wrbyte  flags, psw_reg                        
alu_inc_dec_flags_ret   ret

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
' Op code handlers
do_inr_b                rdbyte  alu, b_reg
                        call    #alu_increment
                        wrbyte  alu, b_reg
                        jmp     #done

do_inr_c                rdbyte  alu, c_reg
                        call    #alu_increment
                        wrbyte  alu, c_reg
                        jmp     #done

do_inr_d                rdbyte  alu, d_reg
                        call    #alu_increment
                        wrbyte  alu, d_reg
                        jmp     #done

do_inr_e                rdbyte  alu, e_reg
                        call    #alu_increment
                        wrbyte  alu, e_reg
                        jmp     #done

do_inr_h                rdbyte  alu, h_reg
                        call    #alu_increment
                        wrbyte  alu, h_reg
                        jmp     #done

do_inr_l                rdbyte  alu, l_reg
                        call    #alu_increment
                        wrbyte  alu, l_reg
                        jmp     #done

do_inr_m                rdword  address, h_reg
                        call    #read_memory_byte
                        mov     alu, data_8
                        call    #alu_increment
                        mov     data_8, alu
                        call    #write_memory_byte
                        jmp     #done

do_inr_a                rdbyte  alu, a_reg
                        call    #alu_increment
                        wrbyte  alu, a_reg
                        jmp     #done


do_dcr_b                rdbyte  alu, b_reg
                        call    #alu_decrement
                        wrbyte  alu, b_reg
                        jmp     #done

do_dcr_c                rdbyte  alu, c_reg
                        call    #alu_decrement
                        wrbyte  alu, c_reg
                        jmp     #done

do_dcr_d                rdbyte  alu, d_reg
                        call    #alu_decrement
                        wrbyte  alu, d_reg
                        jmp     #done

do_dcr_e                rdbyte  alu, e_reg
                        call    #alu_decrement
                        wrbyte  alu, e_reg
                        jmp     #done

do_dcr_h                rdbyte  alu, h_reg
                        call    #alu_decrement
                        wrbyte  alu, h_reg
                        jmp     #done

do_dcr_l                rdbyte  alu, l_reg
                        call    #alu_decrement
                        wrbyte  alu, l_reg
                        jmp     #done

do_dcr_m                rdword  address, h_reg
                        call    #read_memory_byte
                        mov     alu, data_8
                        call    #alu_increment
                        mov     data_8, alu
                        call    #write_memory_byte
                        jmp     #done

do_dcr_a                rdbyte  alu, a_reg
                        call    #alu_decrement
                        wrbyte  alu, a_reg
                        jmp     #done

do_inx_b                rdword  alu, b_reg
                        add     alu, #1
                        wrword  alu, b_reg
                        jmp     #done

do_inx_d                rdword  alu, d_reg
                        add     alu, #1
                        wrword  alu, d_reg
                        jmp     #done

do_inx_h                rdword  alu, h_reg
                        add     alu, #1
                        wrword  alu, h_reg
                        jmp     #done

do_inx_sp               rdword  alu, sp_reg
                        add     alu, #1
                        wrword  alu, sp_reg
                        jmp     #done

do_dcx_b                rdword  alu, b_reg
                        sub     alu, #1
                        wrword  alu, b_reg
                        jmp     #done

do_dcx_d                rdword  alu, d_reg
                        sub     alu, #1
                        wrword  alu, d_reg
                        jmp     #done
 
do_dcx_h                rdword  alu, h_reg
                        sub     alu, #1
                        wrword  alu, h_reg
                        jmp     #done

do_dcx_sp               rdword  alu, sp_reg
                        sub     alu, #1
                        wrword  alu, sp_reg
                        jmp     #done

do_rlc                  rdbyte  alu, a_reg
                        rdbyte  flags, psw_reg
                        shl     alu, #1                    'Shift accumulator left by one
                        test    alu, #%1_0000_0000 wz      'Move any 8 bit carry out bit into acc LSB
                        muxnz   alu, #1
                        muxnz   flags, #carry_bit          'Set carry flag from 8 bit overflow
                        wrbyte  flags, psw_reg
                        wrbyte  alu, a_reg
                        jmp     #done
  
do_rrc                  rdbyte  alu, a_reg
                        rdbyte  flags, psw_reg
                        shr     alu, #1 wc                 'Shift accumulator right by one
                        muxc    alu, #%10000000            'Move props carry to acc. MSB
                        muxc    flags, #carry_bit          'Set carry flag from props carry
                        wrbyte  flags, psw_reg
                        wrbyte  alu, a_reg
                        jmp     #done

do_ral                  rdbyte  alu, a_reg
                        rdbyte  flags, psw_reg
                        shl     alu, #1                    'Shift accumulator left by one
                        test    flags, #carry_bit wz       'Set acc. LSB from carry flag
                        muxnz   alu, #1                        
                        test    alu, #%1_0000_0000 wz      'Set carry flag from 8 bit overflow
                        muxnz   flags, #carry_bit
                        wrbyte  flags, psw_reg
                        wrbyte  alu, a_reg
                        jmp     #done

do_rar                  rdbyte  alu, a_reg
                        rdbyte  flags, psw_reg
                        shr     alu, #1 wc                 'Shift accumulator right by one
                        test    flags, #carry_bit wz       'Set acc. MSB from carry flag
                        muxnz   alu, #%10000000
                        muxc    flags, #carry_bit          'Set carry flag from props carry out
                        wrbyte  flags, psw_reg
                        wrbyte  alu, a_reg
                        jmp     #done

do_lxi_b                call    #read_immediate_word
                        wrword  data_16, b_reg
                        jmp     #done
                         
do_lxi_d                call    #read_immediate_word
                        wrword  data_16, d_reg
                        jmp     #done
                        
do_lxi_h                call    #read_immediate_word
                        wrword  data_16, h_reg
                        jmp     #done
                        
do_lxi_sp               call    #read_immediate_word
                        wrword  data_16, sp_reg
                        jmp     #done
                        
do_mvi_b                call    #read_immediate_byte
                        wrbyte  data_8, b_reg
                        jmp     #done

do_mvi_c                call    #read_immediate_byte
                        wrbyte  data_8, c_reg
                        jmp     #done

do_mvi_d                call    #read_immediate_byte
                        wrbyte  data_8, d_reg
                        jmp     #done

do_mvi_e                call    #read_immediate_byte
                        wrbyte  data_8, e_reg
                        jmp     #done

do_mvi_h                call    #read_immediate_byte
                        wrbyte  data_8, h_reg
                        jmp     #done

do_mvi_l                call    #read_immediate_byte
                        wrbyte  data_8, l_reg
                        jmp     #done

do_mvi_m                call    #read_immediate_byte
                        rdword  address, h_reg
                        call    #write_memory_byte
                        jmp     #done

do_mvi_a                call    #read_immediate_byte
                        wrbyte  data_8, a_reg
                        jmp     #done

do_stax_b               rdword  address, b_reg
                        rdbyte  data_8, a_reg
                        call    #write_memory_byte
                        jmp     #done

do_stax_d               rdword  address, d_reg 
                        rdbyte  data_8, a_reg
                        call    #write_memory_byte
                        jmp     #done

do_ldax_b               rdword  address, b_reg 
                        call    #read_memory_byte
                        wrbyte  data_8, a_reg
                        jmp     #done

do_ldax_d               rdword  address, d_reg
                        call    #read_memory_byte
                        wrbyte  data_8, a_reg
                        jmp     #done


do_dad_b                rdword  operand, b_reg
dad_tail                rdword  alu, h_reg
                        rdbyte  flags, psw_reg
                        add     alu, operand
                        test    alu, mask_bit_16 wz
                        muxnz   flags, #carry_bit
                        wrword  alu, h_reg
                        wrbyte  flags, psw_reg
                        jmp     #done

do_dad_d                rdword  operand, d_reg
                        jmp     #dad_tail

do_dad_h                rdword  operand, h_reg
                        jmp     #dad_tail

do_dad_sp               rdword  operand, sp_reg
                        jmp     #dad_tail

do_daa                  rdbyte  alu, a_reg
                        rdbyte  flags, psw_reg
                        mov     nibble, alu               'Isolate low nibble of accumulator
                        and     nibble, #$0F                        
                        cmp     nibble, #10 wc            'Test for greater than 9                
                        test    flags, #aux_bit wz        'Test auxillary carry
            if_nz_or_nc add     alu,   #$06               'Add 6 to accumulator if greater than 9 or AUX carry
                        add     nibble, #6                'Check for AUX carry from low nibble
                        test    nibble, #%00010000 wz
                        muxnz   flags, #aux_bit           'and set AUX flag if so.      
                        mov     nibble, alu               'Isolate high nibble of accumulator
                        shr     nibble, #4
                        cmp     nibble, #10 wc            'Test for greater than 9
                        test    flags, #carry_bit wz      'Test carry flag
            if_nz_or_nc add     alu, #$60 wc              'Add (6 << 4) to accumulator if greater than 9 or AUX carry
                        test    alu, #$100 wz             'Check for 8 bit carry out
            if_nz       or      flags, #carry_bit         'Set carry flag if so (N.B. Do NOT clear carry if not so)
                        and     alu, #$FF wz, wc
                        muxz    flags, #zero_bit          'Set 8080 zero flag from props zero
                        muxnc   flags, #parity_bit
                        test    alu, #128 wz
                        muxnz   flags, #sign_bit
                        wrbyte  flags, psw_reg
                        wrbyte  alu, a_reg
                        jmp     #done

do_stc                  rdbyte  flags, psw_reg
                        or      flags, #carry_bit         'Set Carry Flag
                        wrbyte  flags, psw_reg
                        jmp     #done
  
do_cmc                  rdbyte  flags, psw_reg
                        xor     flags, #carry_bit         'Complement Carry Flag
                        wrbyte  flags, psw_reg
                        jmp     #done
  
do_cma                  rdbyte  alu, a_reg
                        xor     alu, #$FF
                        wrbyte  alu, a_reg                
                        jmp     #done

do_sta                  call    #read_immediate_word       'Get instructions immediate word into address
                        mov     address, data_16
                        rdbyte  data_8, a_reg
                        call    #write_memory_byte         'Write out the "a" reg to "address"
                        jmp     #done

do_lda                  call    #read_immediate_word       'Get instructions immediate word into address
                        mov     address, data_16
                        call    #read_memory_byte
                        wrbyte  data_8, a_reg
                        jmp     #done

do_shld                 call    #read_immediate_word       'Get instructions immediate word into addess
                        mov     address, data_16
                        rdword  data_16, h_reg
                        call    #write_memory_word
                        jmp     #done

do_lhld                 call    #read_immediate_word       'Get instructions immediate word into address    
                        mov     address, data_16
                        call    #read_memory_word
                        wrword  data_16, h_reg
                        jmp     #done


do_nop                  'FIXME Check there is only one handler for NOP
                        jmp     #done

do_undocumented         'FIXME Checkther is only one handler for these
                        jmp     #done



'---------------------------------------------------------------------------------------------------------
'Local (to this COG) variables
alu                     long    0
operand                 long    0
flags                   long    0
aux                     long    0
nibble                  long    0

mask_bit_16             long    $0001_0000

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Pointers to 8080 registers in HUB RAM
a_reg                   long    0
psw_reg                 long    0
b_reg                   long    0
c_reg                   long    0
d_reg                   long    0
e_reg                   long    0
h_reg                   long    0
l_reg                   long    0
sp_reg                  long    0
pc_reg                  long    0


'93 LONGS free is this much !
long 0[0]
'---------------------------------------------------------------------------------------------------------
                        fit
'---------------------------------------------------------------------------------------------------------
dispatch_table          
{00000000}              word     do_nop          'Cog 0
{00000001}              word     do_lxi_b        'Cog 0
{00000010}              word     do_stax_b       'Cog 0
{00000011}              word     do_inx_b        'Cog 0
{00000100}              word     do_inr_b        'Cog 0
{00000101}              word     do_dcr_b        'Cog 0
{00000110}              word     do_mvi_b        'Cog 0
{00000111}              word     do_rlc          'Cog 0
{00001000}              word     do_undocumented 'Cog 0
{00001001}              word     do_dad_b        'Cog 0
{00001010}              word     do_ldax_b       'Cog 0
{00001011}              word     do_dcx_b        'Cog 0
{00001100}              word     do_inr_c        'Cog 0
{00001101}              word     do_dcr_c        'Cog 0
{00001110}              word     do_mvi_c        'Cog 0
{00001111}              word     do_rrc          'Cog 0
{00010000}              word     do_undocumented 'Cog 0
{00010001}              word     do_lxi_d        'Cog 0
{00010010}              word     do_stax_d       'Cog 0
{00010011}              word     do_inx_d        'Cog 0
{00010100}              word     do_inr_d        'Cog 0
{00010101}              word     do_dcr_d        'Cog 0
{00010110}              word     do_mvi_d        'Cog 0
{00010111}              word     do_ral          'Cog 0
{00011000}              word     do_undocumented 'Cog 0
{00011001}              word     do_dad_d        'Cog 0
{00011010}              word     do_ldax_d       'Cog 0
{00011011}              word     do_dcx_d        'Cog 0
{00011100}              word     do_inr_e        'Cog 0
{00011101}              word     do_dcr_e        'Cog 0
{00011110}              word     do_mvi_e        'Cog 0
{00011111}              word     do_rar          'Cog 0
{00100000}              word     do_undocumented 'Cog 0
{00100001}              word     do_lxi_h        'Cog 0
{00100010}              word     do_shld         'Cog 0
{00100011}              word     do_inx_h        'Cog 0
{00100100}              word     do_inr_h        'Cog 0
{00100101}              word     do_dcr_h        'Cog 0
{00100110}              word     do_mvi_h        'Cog 0
{00100111}              word     do_daa          'Cog 0
{00101000}              word     do_undocumented 'Cog 0
{00101001}              word     do_dad_h        'Cog 0
{00101010}              word     do_lhld         'Cog 0
{00101011}              word     do_dcx_h        'Cog 0
{00101100}              word     do_inr_l        'Cog 0
{00101101}              word     do_dcr_l        'Cog 0
{00101110}              word     do_mvi_l        'Cog 0
{00101111}              word     do_cma          'Cog 0
{00110000}              word     do_undocumented 'Cog 0
{00110001}              word     do_lxi_sp       'Cog 0
{00110010}              word     do_sta          'Cog 0
{00110011}              word     do_inx_sp       'Cog 0
{00110100}              word     do_inr_m        'Cog 0
{00110101}              word     do_dcr_m        'Cog 0
{00110110}              word     do_mvi_m        'Cog 0
{00110111}              word     do_stc          'Cog 0
{00111000}              word     do_undocumented 'Cog 0
{00111001}              word     do_dad_sp       'Cog 0
{00111010}              word     do_lda          'Cog 0
{00111011}              word     do_dcx_sp       'Cog 0
{00111100}              word     do_inr_a        'Cog 0
{00111101}              word     do_dcr_a        'Cog 0
{00111110}              word     do_mvi_a        'Cog 0
{00111111}              word     do_cmc          'Cog 0

'All the mov instructions

{01000000}              word    skip            'do_mov_b_b 'Cog 1
{01000001}              word    skip            'do_mov_b_c 'Cog 1
{01000010}              word    skip            'do_mov_b_d 'Cog 1
{01000011}              word    skip            'do_mov_b_e 'Cog 1
{01000100}              word    skip            'do_mov_b_h 'Cog 1
{01000101}              word    skip            'do_mov_b_l 'Cog 1
{01000110}              word    skip            'do_mov_b_m 'Cog 1
{01000111}              word    skip            'do_mov_b_a 'Cog 1
{01001000}              word    skip            'do_mov_c_b 'Cog 1
{01001001}              word    skip            'do_mov_c_c 'Cog 1
{01001010}              word    skip            'do_mov_c_d 'Cog 1
{01001011}              word    skip            'do_mov_c_e 'Cog 1
{01001100}              word    skip            'do_mov_c_h 'Cog 1
{01001101}              word    skip            'do_mov_c_l 'Cog 1
{01001110}              word    skip            'do_mov_c_m 'Cog 1
{01001111}              word    skip            'do_mov_c_a 'Cog 1
{01010000}              word    skip            'do_mov_d_b 'Cog 1
{01010001}              word    skip            'do_mov_d_c 'Cog 1
{01010010}              word    skip            'do_mov_d_d 'Cog 1
{01010011}              word    skip            'do_mov_d_e 'Cog 1
{01010100}              word    skip            'do_mov_d_h 'Cog 1
{01010101}              word    skip            'do_mov_d_l 'Cog 1
{01010110}              word    skip            'do_mov_d_m 'Cog 1
{01010111}              word    skip            'do_mov_d_a 'Cog 1
{01011000}              word    skip            'do_mov_e_b 'Cog 1
{01011001}              word    skip            'do_mov_e_c 'Cog 1
{01011010}              word    skip            'do_mov_e_d 'Cog 1
{01011011}              word    skip            'do_mov_e_e 'Cog 1
{01011100}              word    skip            'do_mov_e_h 'Cog 1
{01011101}              word    skip            'do_mov_e_l 'Cog 1
{01011110}              word    skip            'do_mov_e_m 'Cog 1
{01011111}              word    skip            'do_mov_e_a 'Cog 1
{01100000}              word    skip            'do_mov_h_b 'Cog 1
{01100001}              word    skip            'do_mov_h_c 'Cog 1
{01100010}              word    skip            'do_mov_h_d 'Cog 1
{01100011}              word    skip            'do_mov_h_e 'Cog 1
{01100100}              word    skip            'do_mov_h_h 'Cog 1
{01100101}              word    skip            'do_mov_h_l 'Cog 1
{01100110}              word    skip            'do_mov_h_m 'Cog 1
{01100111}              word    skip            'do_mov_h_a 'Cog 1
{01101000}              word    skip            'do_mov_l_b 'Cog 1
{01101001}              word    skip            'do_mov_l_c 'Cog 1
{01101010}              word    skip            'do_mov_l_d 'Cog 1
{01101011}              word    skip            'do_mov_l_e 'Cog 1
{01101100}              word    skip            'do_mov_l_h 'Cog 1
{01101101}              word    skip            'do_mov_l_l 'Cog 1
{01101110}              word    skip            'do_mov_l_m 'Cog 1
{01101111}              word    skip            'do_mov_l_a 'Cog 1
{01110000}              word    skip            'do_mov_m_b 'Cog 1
{01110001}              word    skip            'do_mov_m_c 'Cog 1
{01110010}              word    skip            'do_mov_m_d 'Cog 1
{01110011}              word    skip            'do_mov_m_e 'Cog 1
{01110100}              word    skip            'do_mov_m_h 'Cog 1
{01110101}              word    skip            'do_mov_m_l 'Cog 1
{01110110}              word    skip            'do_halt 'Cog 1
{01110111}              word    skip            'do_mov_m_a 'Cog 1
{01111000}              word    skip            'do_mov_a_b 'Cog 1
{01111001}              word    skip            'do_mov_a_c 'Cog 1
{01111010}              word    skip            'do_mov_a_d 'Cog 1
{01111011}              word    skip            'do_mov_a_e 'Cog 1
{01111100}              word    skip            'do_mov_a_h 'Cog 1
{01111101}              word    skip            'do_mov_a_l 'Cog 1
{01111110}              word    skip            'do_mov_a_m 'Cog 1
{01111111}              word    skip            'do_mov_a_a 'Cog 1

                        

'All the reg/mem to accumulator instructions
{10000000}              word    skip            'do_add_b 'Cog 2
{10000001}              word    skip            'do_add_c 'Cog 2
{10000010}              word    skip            'do_add_d 'Cog 2
{10000011}              word    skip            'do_add_e 'Cog 2
{10000100}              word    skip            'do_add_h 'Cog 2
{10000101}              word    skip            'do_add_l 'Cog 2
{10000110}              word    skip            'do_add_m 'Cog 2
{10000111}              word    skip            'do_add_a 'Cog 2
{10001000}              word    skip            'do_adc_b 'Cog 2
{10001001}              word    skip            'do_adc_c 'Cog 2
{10001010}              word    skip            'do_adc_d 'Cog 2
{10001011}              word    skip            'do_adc_e 'Cog 2
{10001100}              word    skip            'do_adc_h 'Cog 2
{10001101}              word    skip            'do_adc_l 'Cog 2
{10001110}              word    skip            'do_adc_m 'Cog 2
{10001111}              word    skip            'do_adc_a 'Cog 2
{10010000}              word    skip            'do_sub_b 'Cog 2
{10010001}              word    skip            'do_sub_c 'Cog 2
{10010010}              word    skip            'do_sub_d 'Cog 2
{10010011}              word    skip            'do_sub_e 'Cog 2
{10010100}              word    skip            'do_sub_h 'Cog 2
{10010101}              word    skip            'do_sub_l 'Cog 2
{10010110}              word    skip            'do_sub_m 'Cog 2
{10010111}              word    skip            'do_sub_a 'Cog 2
{10011000}              word    skip            'do_sbb_b 'Cog 2
{10011001}              word    skip            'do_sbb_c 'Cog 2
{10011010}              word    skip            'do_sbb_d 'Cog 2
{10011011}              word    skip            'do_sbb_e 'Cog 2
{10011100}              word    skip            'do_sbb_h 'Cog 2
{10011101}              word    skip            'do_sbb_l 'Cog 2
{10011110}              word    skip            'do_sbb_m 'Cog 2
{10011111}              word    skip            'do_sbb_a 'Cog 2

{10100000}              word    skip            'do_ana_b 'Cog 2
{10100001}              word    skip            'do_ana_c 'Cog 2
{10100010}              word    skip            'do_ana_d 'Cog 2
{10100011}              word    skip            'do_ana_e 'Cog 2
{10100100}              word    skip            'do_ana_h 'Cog 2
{10100101}              word    skip            'do_ana_l 'Cog 2
{10100110}              word    skip            'do_ana_m 'Cog 2
{10100111}              word    skip            'do_ana_a 'Cog 2

{10101000}              word    skip            'do_xra_b 'Cog 2
{10101001}              word    skip            'do_xra_c 'Cog 2
{10101010}              word    skip            'do_xra_d 'Cog 2
{10101011}              word    skip            'do_xra_e 'Cog 2
{10101100}              word    skip            'do_xra_h 'Cog 2
{10101101}              word    skip            'do_xra_l 'Cog 2
{10101110}              word    skip            'do_xra_m 'Cog 2
{10101111}              word    skip            'do_xra_a 'Cog 2

{10110000}              word    skip            'do_ora_b 'Cog 2
{10110001}              word    skip            'do_ora_c 'Cog 2
{10110010}              word    skip            'do_ora_d 'Cog 2
{10110011}              word    skip            'do_ora_e 'Cog 2
{10110100}              word    skip            'do_ora_h 'Cog 2
{10110101}              word    skip            'do_ora_l 'Cog 2
{10110110}              word    skip            'do_ora_m 'Cog 2
{10110111}              word    skip            'do_ora_a 'Cog 2

{10111000}              word    skip            'do_cmp_b 'Cog 2
{10111001}              word    skip            'do_cmp_c 'Cog 2
{10111010}              word    skip            'do_cmp_d 'Cog 2
{10111011}              word    skip            'do_cmp_e 'Cog 2
{10111100}              word    skip            'do_cmp_h 'Cog 2
{10111101}              word    skip            'do_cmp_l 'Cog 2
{10111110}              word    skip            'do_cmp_m 'Cog 2
{10111111}              word    skip            'do_cmp_a 'Cog 2
                        
{11000000}              word    skip            'do_rnz 'Cog 3
{11000001}              word    skip            'do_pop_b 'Cog 3
{11000010}              word    skip            'do_jnz 'Cog 3
{11000011}              word    skip            'do_jmp 'Cog 3
{11000100}              word    skip            'do_cnz 'Cog 3
{11000101}              word    skip            'do_push_b 'Cog 3
{11000110}              word    skip            'do_adi 'Cog 2
{11000111}              word    skip            'do_rst_0 'Cog 3
{11001000}              word    skip            'do_rz 'Cog 3
{11001001}              word    skip            'do_ret 'Cog 3
{11001010}              word    skip            'do_jz 'Cog 3
{11001011}              word    do_undocumented   'Cog 0
{11001100}              word    skip            'do_cz 'Cog 3
{11001101}              word    skip            'do_call 'Cog 3
{11001110}              word    skip            'do_aci 'Cog 2
{11001111}              word    skip            'do_rst_1 'Cog 3
{11010000}              word    skip            'do_rnc 'Cog 3
{11010001}              word    skip            'do_pop_d 'Cog 3
{11010010}              word    skip            'do_jnc 'Cog 3
{11010011}              word    skip            'do_out 'Cog 3
{11010100}              word    skip            'do_cnc 'Cog 3
{11010101}              word    skip            'do_push_d 'Cog 3
{11010110}              word    skip            'do_sui 'Cog 2
{11010111}              word    skip            'do_rst_2 'Cog 3
{11011000}              word    skip            'do_rc 'Cog 3
{11011001}              word    do_undocumented 'Cog 0
{11011010}              word    skip            'do_jc 'Cog 3
{11011011}              word    skip            'do_in 'Cog 3
{11011100}              word    skip            'do_cc 'Cog 3
{11011101}              word    do_undocumented 'Cog 0
{11011110}              word    skip            'do_sbi 'Cog 2
{11011111}              word    skip            'do_rst_3 'Cog 3
{11100000}              word    skip            'do_rpo 'Cog 3
{11100001}              word    skip            'do_pop_h 'Cog 3
{11100010}              word    skip            'do_jpo 'Cog 3
{11100011}              word    skip            'do_xthl 'Cog 3
{11100100}              word    skip            'do_cpo 'Cog 3
{11100101}              word    skip            'do_push_h 'Cog 3
{11100110}              word    skip            'do_ani 'Cog 2
{11100111}              word    skip            'do_rst_4 'Cog 3
{11101000}              word    skip            'do_rpe 'Cog 3
{11101001}              word    skip            'do_pchl 'Cog 3
{11101010}              word    skip            'do_jpe 'Cog 3
{11101011}              word    skip            'do_xchg 'Cog 3
{11101100}              word    skip            'do_cpe 'Cog 3
{11101101}              word    do_undocumented 'Cog 0
{11101110}              word    skip            'do_xri 'Cog 2
{11101111}              word    skip            'do_rst_5 'Cog 3
{11110000}              word    skip            'do_rp 'Cog 3
{11110001}              word    skip            'do_pop_af 'Cog 3
{11110010}              word    skip            'do_jp 'Cog 3
{11110011}              word    skip            'do_di 'Cog 3
{11110100}              word    skip            'do_cp 'Cog 3
{11110101}              word    skip            'do_push_af 'Cog 3
{11110110}              word    skip            'do_ori 'Cog 2
{11110111}              word    skip            'do_rst_6 'Cog 3
{11111000}              word    skip            'do_rm 'Cog 3
{11111001}              word    skip            'do_sphl 'Cog 3
{11111010}              word    skip            'do_jm 'Cog 3
{11111011}              word    skip            'do_ei 'Cog 3
{11111100}              word    skip            'do_cm 'Cog 3
{11111101}              word    do_undocumented 'Cog 0
{11111110}              word    skip            'do_cpi 'Cog 2
{11111111}              word    skip            'do_rst_7 'Cog 3
'---------------------------------------------------------------------------------------------------------