'*************************************
'*  i8080_emu v0.0                   *
'*  An emulation of the Intel i8080  *
'*  (C) 2009 Michael Rychlik         *
'*************************************
'
'v0.0 - 27 Jan 2009 - Original version

CON

  '8080 Program Status Word bits
  sign_bit   = %10000000
  zero_bit   = %01000000
  aux_bit    = %00010000
  parity_bit = %00000100
  carry_bit  = %00000001

VAR
  long  cog

PUB start(cpu_params) : okay

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
''   cpu_par_params is a pointer to CPU parameter list

  object_base := @@0                            'Set up the base address of this object
  
  stop
  okay := cog := cognew(@enter, cpu_params) + 1 'Start emulator in a new COG

PUB stop

'' Stop CPU simulator - frees a cog

  if cog
    cogstop(cog~ - 1)

DAT
                        org     0

enter                   mov     cpu_params_ptr, par
                        rdlong  reg_ptr, cpu_params_ptr    'First item in CPU params list is 
                        mov     psw_reg, reg_ptr           'pointer to register file.
                        add     reg_ptr, #1
                        mov     a_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     c_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     b_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     e_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     d_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     l_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     h_reg, reg_ptr
                        add     reg_ptr, #1
                        mov     sp_reg, reg_ptr
                        add     reg_ptr, #2
                        mov     pc_reg, reg_ptr
                        add     cpu_params_ptr, #4         'Move parAms pointer up to ram_base
                        rdlong  ram_base, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to ram_len
                        rdlong  ram_size, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to rom base
                        rdlong  rom_base, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to rom address
                        rdlong  rom_address, cpu_params_ptr
                        add     cpu_params_ptr, #4         'Move params pointer up to I/O interface add
                                                           'FIXME not implemented yet
                        add     cpu_params_ptr, #4         'Move params pointer up CPU control address
                        rdlong  cpu_control, cpu_params_ptr


'---------------------------------------------------------------------------------------------------------
' Instruction fetch and execute loop. Common to all CPU emulator COGS

                        add     dispatch_tab_addr, object_base 'Fixup this object offset to HUB real address
                        
                        rdword  pc, pc_reg                 'Initialize COG program counter from global 
                        jmp     #fetch                     'program counter and fetch first instruction

done                    wrword  pc, pc_reg                 'Instruction executed so up date global (HUB)
                                                           'Program Counter and fetch next instruction

                        mov     data_16, #0                '\ DEBUG CODE
                        wrword  data_16, cpu_control       '/

                                                           'Instruction fetch loop
fetch                   call    #read_immediate_byte       'Get the instruction into 'data_8'

halt                    rdword  data_16, cpu_control       '\
                        cmp     data_16, #0 wz             ' | DEBUG CODE 
              if_z      jmp     #halt                      '/

                        mov     scratch, data_8            'Dispatch table look up
                        shl     scratch, #1                'Mul by 2 to get a WORD offset into the table
                        add     scratch, dispatch_tab_addr 'Index into table
                        rdword  scratch, scratch           'Read the instruction handlers address
                        jmp     scratch                    'Jump to instruction handler

skip                    rdword  pc_next, pc_reg            'Wait until global (HUB) Program Counter
                        cmp     pc_next, pc                'chanages. I.E. another COG exexuted this op.
              if_z      jmp     #skip
                        mov     pc, pc_next                'PC changed so lets go
                        jmp     #fetch

scratch                 long    0
dispatch_tab_addr       long    @dispatch_table
object_base             long    0-0                        'HUB address of this object 
cpu_params_ptr          long    0
reg_ptr                 long    0
pc                      long    0                          'Local COG program conter
pc_next                 long    0

'---------------------------------------------------------------------------------------------------------
cpu_control             long    0
'---------------------------------------------------------------------------------------------------------
'Functions common (duplicated in) all CPU emulator COGs


'Read a BYTE from location given by "address" of 8080 RAM into byte data_8
read_memory_byte        cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_b      jmp     #:read_ram
                        call    #read_rom
                        jmp     #read_memory_byte_ret
:read_ram               mov     hub_pointer, ram_base      'Point to the RAM
                        add     hub_pointer, address       'Add address offset
                        rdbyte  data_8, hub_pointer        'Read the byte from there
read_memory_byte_ret    ret


read_rom                cmp     address, rom_start wc,wz   'Is the given address within the ROM range ?
              if_b      jmp     #read_rom_ret              'address below rom_base => return with "data" undefined
                        mov     rom_address, address
                        sub     rom_address, rom_start     'address becomes offset into ROM from start of ROM                  
                        mov     hub_pointer, rom_base      'Point to the ROM
                        add     hub_pointer, rom_address   'Add address offset                     
                        rdbyte  data_8, hub_pointer        'Read the byte from there
read_rom_ret            ret                                'Return


'Write the BYTE data_8 to location given by "address"
write_memory_byte       cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_b      mov     hub_pointer, ram_base      'Point to the RAM
              if_b      add     hub_pointer, address       'Add address offset
              if_b      wrbyte  data_8, hub_pointer        'Write the byte there
write_memory_byte_ret   ret


'Read a WORD from location given by "address" of 8080 RAM into byte data_16
read_memory_word        call    #read_memory_byte          'Read the low byte
                        mov     data_16, data_8
                        ror     data_16, #8
                        add     address, #1
                        call    #read_memory_byte          'Read the high byte
                        or      data_16, data_8
                        rol     data_16, #8
read_memory_word_ret    ret

'Write the WORD data_16 to location given by "address"
write_memory_word       mov     data_8, data_16
                        call    #write_memory_byte         'Write the LOW byte
                        shr     data_8, #8
                        add     address, #1
                        call    #write_memory_byte
write_memory_word_ret   ret

'The following configuration "constants" set from COG PARameters.
ram_base                long    0-0                        'HUB address of the 8080 RAM area
ram_size                long    0-0                        'Length in bytes of available 8080 RAM
rom_base                long    0-0                        'HUB address of the BOOT ROM image            
rom_start               long    0-0                        'ROM starts at this address in 8080 space
rom_address             long    0-0

'8080 RAM read/write vars.
hub_pointer             long    0                          'Pointer into HUB RAM
address                 long    0                          'Address param for read/write funcs
data_8                  long    0                          'Data bytes read into here
data_16                 long    0                          'Data words read into here



'Read a BYTE from location given by "pc" of 8080 RAM into byte data_8 and increment pc
read_immediate_byte     mov     address, pc
                        call    #read_memory_byte
                        add     pc, #1
read_immediate_byte_ret ret

'Read a WORD from location given by "pc" of 8080 RAM into byte data_8 and pc = pc + 2
read_immediate_word     mov     address, pc
                        call    #read_memory_word
                        add     pc, #2
read_immediate_word_ret ret
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Functions unique to this COG

push                    'FIXME
push_ret                ret

pop                     'FIXME
pop_ret                 ret
'---------------------------------------------------------------------------------------------------------



'---------------------------------------------------------------------------------------------------------
' Op code handlers

' For POP the sequence is:
' 1. Read LOW byte to stack at [SP]
' 2. Increment the stack pointer
' 1. Read HIGH byte to stack at [SP]
' 3. Increment the stack pointer

do_pop_b                call    #pop
                        wrword  alu, b_reg
                        jmp     #done

do_pop_d                call    #pop
                        wrword  alu, d_reg
                        jmp     #done

do_pop_h                call    #pop
                        wrword  alu, h_reg
                        jmp     #done

do_pop_af               call    #pop
                        wrword  alu, a_reg
                        jmp     #done

' For PUSH the sequence is:
' 1. Decrement the stack pointer
' 2. Write HIGH byte to stack at [SP]
' 3. Decrement the stack pointer
' 4. Write the LOW byte to stack at [SP]

do_push_b               rdword  alu, b_reg
                        call    #push
                        jmp     #done

do_push_d               rdword  alu, d_reg
                        call    #push
                        jmp     #done

do_push_h               rdword  alu, h_reg
                        call    #push
                        jmp     #done

do_push_af              rdword  alu, a_reg
                        call    #push
                        jmp     #done

do_jnz                  rdbyte  flags, psw_reg
                        test    flags, #zero_bit wz
              if_nz     jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                
do_jz                   rdbyte  flags, psw_reg
                        test    flags, #zero_bit wz
              if_z      jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                
do_jmp                  call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done

do_jnc                  rdbyte  flags, psw_reg
                        test    flags, #carry_bit wz
              if_nz     jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                
                                   
do_jc                   rdbyte  flags, psw_reg
                        test    flags, #carry_bit wz
              if_z      jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                
do_jpo                  rdbyte  flags, psw_reg
                        test    flags, #parity_bit wz
              if_nz     jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                 
do_jpe                  rdbyte  flags, psw_reg
                        test    flags, #parity_bit wz
              if_z      jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                
do_jp                   rdbyte  flags, psw_reg
                        test    flags, #sign_bit wz
              if_nz     jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                
do_jm                   rdbyte  flags, psw_reg
                        test    flags, #sign_bit wz
              if_z      jmp     #:no_jump
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_jump                add     pc, #2
                        jmp     #done
                

do_rnz                  rdbyte  flags, psw_reg
                        test    flags, #carry_bit wz
              if_nz     jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done
                        
                                   
do_rz                   rdbyte  flags, psw_reg
                        test    flags, #zero_bit wz
              if_z      jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done
                        

do_ret                  call    #pop
                        mov     pc, alu
                        jmp     #done

do_rnc                  rdbyte  flags, psw_reg
                        test    flags, #carry_bit wz
              if_z      jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done
                        
do_rc                   rdbyte  flags, psw_reg
                        test    flags, #carry_bit wz
              if_nz     jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done
                        
do_rpo                  rdbyte  flags, psw_reg
                        test    flags, #parity_bit wz
              if_nz     jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done

do_rpe                  rdbyte  flags, psw_reg
                        test    flags, #parity_bit wz
              if_z      jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done

do_rp                   rdbyte  flags, psw_reg
                        test    flags, #sign_bit wz
              if_nz     jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done

do_rm                   rdbyte  flags, psw_reg
                        test    flags, #sign_bit wz
              if_z      jmp     #:no_ret
                        call    #pop
                        mov     pc, alu
                        jmp     #done
:no_ret                 jmp     #done
                        

do_cnz                  rdbyte  flags, psw_reg
                        test    flags, #zero_bit wz
              if_nz     jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done
                        


do_cz                   rdbyte  flags, psw_reg
                        test    flags, #zero_bit wz
              if_z      jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done
                        

do_call                 mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done


do_cnc                  rdbyte  flags, psw_reg
                        test    flags, #carry_bit wz
              if_z      jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done
                        
do_cc                   rdbyte  flags, psw_reg
                        test    flags, #carry_bit wz
              if_nz     jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done
                        
do_cpo                  rdbyte  flags, psw_reg
                        test    flags, #parity_bit wz
              if_z      jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done
                        
do_cpe                  rdbyte  flags, psw_reg
                        test    flags, #parity_bit wz
              if_nz     jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done
                        

do_cp                   rdbyte  flags, psw_reg
                        test    flags, #sign_bit wz
              if_z      jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done
                        
do_cm                   rdbyte  flags, psw_reg
                        test    flags, #sign_bit wz
              if_nz     jmp     #:no_call
                        mov     alu, pc
                        add     alu, #2
                        call    #push
                        call    #read_immediate_word
                        mov     pc, data_16
                        jmp     #done
:no_call                add     pc, #2
                        jmp     #done

do_rst_0                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0000
                        wrword  alu, pc_reg
                        jmp     #done
                        
do_rst_1                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0008
                        wrword  alu, pc_reg
                        jmp     #done
                        
do_rst_2                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0010
                        wrword  alu, pc_reg
                        jmp     #done
                        
do_rst_3                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0018
                        wrword  alu, pc_reg
                        jmp     #done
                        
do_rst_4                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0020
                        wrword  alu, pc_reg
                        jmp     #done
                        
do_rst_5                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0028
                        wrword  alu, pc_reg
                        jmp     #done
                        
do_rst_6                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0030
                        wrword  alu, pc_reg
                        jmp     #done
                        
do_rst_7                rdword  alu, pc_reg
                        call    #push
                        mov     alu, #$0038
                        wrword  alu, pc_reg
                        jmp     #done
                        


do_out                                   
do_in                                   

do_xthl                 

do_pchl                                   

do_xchg

do_sphl                                   

do_di                   jmp     #done
  
do_ei                   jmp     #done
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Local (to this COG) variables
alu                     long    0
flags                   long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Pointers to 8080 registers in HUB RAM
a_reg                   long    0
psw_reg                 long    0
b_reg                   long    0
c_reg                   long    0
d_reg                   long    0
e_reg                   long    0
h_reg                   long    0
l_reg                   long    0
sp_reg                  long    0
pc_reg                  long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------

long 0[0]
                        fit
'---------------------------------------------------------------------------------------------------------
dispatch_table          
{00000000}              word     skip            'do_nop 'Cog 0
{00000001}              word     skip            'do_lxi_b 'Cog 0
{00000010}              word     skip            'do_stax_b 'Cog 0
{00000011}              word     skip            'do_inx_b 'Cog 0
{00000100}              word     skip            'do_inr_b 'Cog 0
{00000101}              word     skip            'do_dcr_b 'Cog 0
{00000110}              word     skip            'do_mvi_b 'Cog 0
{00000111}              word     skip            'do_rlc 'Cog 0
{00001000}              word     skip            'do_undocumented 'Cog 0
{00001001}              word     skip            'do_dad_b 'Cog 0
{00001010}              word     skip            'do_ldax_b 'Cog 0
{00001011}              word     skip            'do_dcx_b 'Cog 0
{00001100}              word     skip            'do_inr_c 'Cog 0
{00001101}              word     skip            'do_dcr_c 'Cog 0
{00001110}              word     skip            'do_mvi_c 'Cog 0
{00001111}              word     skip            'do_rrc 'Cog 0
{00010000}              word     skip            'do_undocumented 'Cog 0
{00010001}              word     skip            'do_lxi_d 'Cog 0
{00010010}              word     skip            'do_stax_d 'Cog 0
{00010011}              word     skip            'do_inx_d 'Cog 0
{00010100}              word     skip            'do_inr_d 'Cog 0
{00010101}              word     skip            'do_dcr_d 'Cog 0
{00010110}              word     skip            'do_mvi_d 'Cog 0
{00010111}              word     skip            'do_ral 'Cog 0
{00011000}              word     skip            'do_undocumented 'Cog 0
{00011001}              word     skip            'do_dad_d 'Cog 0
{00011010}              word     skip            'do_ldax_d 'Cog 0
{00011011}              word     skip            'do_dcx_d 'Cog 0
{00011100}              word     skip            'do_inr_e 'Cog 0
{00011101}              word     skip            'do_dcr_e 'Cog 0
{00011110}              word     skip            'do_mvi_e 'Cog 0
{00011111}              word     skip            'do_rar 'Cog 0
{00100000}              word     skip            'do_undocumented 'Cog 0
{00100001}              word     skip            'do_lxi_h 'Cog 0
{00100010}              word     skip            'do_shld 'Cog 0
{00100011}              word     skip            'do_inx_h 'Cog 0
{00100100}              word     skip            'do_inr_h 'Cog 0
{00100101}              word     skip            'do_dcr_h 'Cog 0
{00100110}              word     skip            'do_mvi_h 'Cog 0
{00100111}              word     skip            'do_daa 'Cog 0
{00101000}              word     skip            'do_undocumented 'Cog 0
{00101001}              word     skip            'do_dad_h 'Cog 0
{00101010}              word     skip            'do_lhld 'Cog 0
{00101011}              word     skip            'do_dcx_h 'Cog 0
{00101100}              word     skip            'do_inr_l 'Cog 0
{00101101}              word     skip            'do_dcr_l 'Cog 0
{00101110}              word     skip            'do_mvi_l 'Cog 0
{00101111}              word     skip            'do_cma 'Cog 0
{00110000}              word     skip            'do_undocumented 'Cog 0
{00110001}              word     skip            'do_lxi_sp 'Cog 0
{00110010}              word     skip            'do_sta 'Cog 0
{00110011}              word     skip            'do_inx_sp 'Cog 0
{00110100}              word     skip            'do_inr_m 'Cog 0
{00110101}              word     skip            'do_dcr_m 'Cog 0
{00110110}              word     skip            'do_mvi_m 'Cog 0
{00110111}              word     skip            'do_stc 'Cog 0
{00111000}              word     skip            'do_undocumented 'Cog 0
{00111001}              word     skip            'do_dad_sp 'Cog 0
{00111010}              word     skip            'do_lda 'Cog 0
{00111011}              word     skip            'do_dcx_sp 'Cog 0
{00111100}              word     skip            'do_inr_a 'Cog 0
{00111101}              word     skip            'do_dcr_a 'Cog 0
{00111110}              word     skip            'do_mvi_a 'Cog 0
{00111111}              word     skip            'do_cmc 'Cog 0

'All the mov instructions

{01000000}              word    skip            'do_mov_b_b 'Cog 1
{01000001}              word    skip            'do_mov_b_c 'Cog 1
{01000010}              word    skip            'do_mov_b_d 'Cog 1
{01000011}              word    skip            'do_mov_b_e 'Cog 1
{01000100}              word    skip            'do_mov_b_h 'Cog 1
{01000101}              word    skip            'do_mov_b_l 'Cog 1
{01000110}              word    skip            'do_mov_b_m 'Cog 1
{01000111}              word    skip            'do_mov_b_a 'Cog 1
{01001000}              word    skip            'do_mov_c_b 'Cog 1
{01001001}              word    skip            'do_mov_c_c 'Cog 1
{01001010}              word    skip            'do_mov_c_d 'Cog 1
{01001011}              word    skip            'do_mov_c_e 'Cog 1
{01001100}              word    skip            'do_mov_c_h 'Cog 1
{01001101}              word    skip            'do_mov_c_l 'Cog 1
{01001110}              word    skip            'do_mov_c_m 'Cog 1
{01001111}              word    skip            'do_mov_c_a 'Cog 1
{01010000}              word    skip            'do_mov_d_b 'Cog 1
{01010001}              word    skip            'do_mov_d_c 'Cog 1
{01010010}              word    skip            'do_mov_d_d 'Cog 1
{01010011}              word    skip            'do_mov_d_e 'Cog 1
{01010100}              word    skip            'do_mov_d_h 'Cog 1
{01010101}              word    skip            'do_mov_d_l 'Cog 1
{01010110}              word    skip            'do_mov_d_m 'Cog 1
{01010111}              word    skip            'do_mov_d_a 'Cog 1
{01011000}              word    skip            'do_mov_e_b 'Cog 1
{01011001}              word    skip            'do_mov_e_c 'Cog 1
{01011010}              word    skip            'do_mov_e_d 'Cog 1
{01011011}              word    skip            'do_mov_e_e 'Cog 1
{01011100}              word    skip            'do_mov_e_h 'Cog 1
{01011101}              word    skip            'do_mov_e_l 'Cog 1
{01011110}              word    skip            'do_mov_e_m 'Cog 1
{01011111}              word    skip            'do_mov_e_a 'Cog 1
{01100000}              word    skip            'do_mov_h_b 'Cog 1
{01100001}              word    skip            'do_mov_h_c 'Cog 1
{01100010}              word    skip            'do_mov_h_d 'Cog 1
{01100011}              word    skip            'do_mov_h_e 'Cog 1
{01100100}              word    skip            'do_mov_h_h 'Cog 1
{01100101}              word    skip            'do_mov_h_l 'Cog 1
{01100110}              word    skip            'do_mov_h_m 'Cog 1
{01100111}              word    skip            'do_mov_h_a 'Cog 1
{01101000}              word    skip            'do_mov_l_b 'Cog 1
{01101001}              word    skip            'do_mov_l_c 'Cog 1
{01101010}              word    skip            'do_mov_l_d 'Cog 1
{01101011}              word    skip            'do_mov_l_e 'Cog 1
{01101100}              word    skip            'do_mov_l_h 'Cog 1
{01101101}              word    skip            'do_mov_l_l 'Cog 1
{01101110}              word    skip            'do_mov_l_m 'Cog 1
{01101111}              word    skip            'do_mov_l_a 'Cog 1
{01110000}              word    skip            'do_mov_m_b 'Cog 1
{01110001}              word    skip            'do_mov_m_c 'Cog 1
{01110010}              word    skip            'do_mov_m_d 'Cog 1
{01110011}              word    skip            'do_mov_m_e 'Cog 1
{01110100}              word    skip            'do_mov_m_h 'Cog 1
{01110101}              word    skip            'do_mov_m_l 'Cog 1
{01110110}              word    skip            'do_halt 'Cog 1
{01110111}              word    skip            'do_mov_m_a 'Cog 1
{01111000}              word    skip            'do_mov_a_b 'Cog 1
{01111001}              word    skip            'do_mov_a_c 'Cog 1
{01111010}              word    skip            'do_mov_a_d 'Cog 1
{01111011}              word    skip            'do_mov_a_e 'Cog 1
{01111100}              word    skip            'do_mov_a_h 'Cog 1
{01111101}              word    skip            'do_mov_a_l 'Cog 1
{01111110}              word    skip            'do_mov_a_m 'Cog 1
{01111111}              word    skip            'do_mov_a_a 'Cog 1

                        

'All the reg/mem to accumulator instructions
{10000000}              word    skip            'do_add_b 'Cog 2
{10000001}              word    skip            'do_add_c 'Cog 2
{10000010}              word    skip            'do_add_d 'Cog 2
{10000011}              word    skip            'do_add_e 'Cog 2
{10000100}              word    skip            'do_add_h 'Cog 2
{10000101}              word    skip            'do_add_l 'Cog 2
{10000110}              word    skip            'do_add_m 'Cog 2
{10000111}              word    skip            'do_add_a 'Cog 2
{10001000}              word    skip            'do_adc_b 'Cog 2
{10001001}              word    skip            'do_adc_c 'Cog 2
{10001010}              word    skip            'do_adc_d 'Cog 2
{10001011}              word    skip            'do_adc_e 'Cog 2
{10001100}              word    skip            'do_adc_h 'Cog 2
{10001101}              word    skip            'do_adc_l 'Cog 2
{10001110}              word    skip            'do_adc_m 'Cog 2
{10001111}              word    skip            'do_adc_a 'Cog 2
{10010000}              word    skip            'do_sub_b 'Cog 2
{10010001}              word    skip            'do_sub_c 'Cog 2
{10010010}              word    skip            'do_sub_d 'Cog 2
{10010011}              word    skip            'do_sub_e 'Cog 2
{10010100}              word    skip            'do_sub_h 'Cog 2
{10010101}              word    skip            'do_sub_l 'Cog 2
{10010110}              word    skip            'do_sub_m 'Cog 2
{10010111}              word    skip            'do_sub_a 'Cog 2
{10011000}              word    skip            'do_sbb_b 'Cog 2
{10011001}              word    skip            'do_sbb_c 'Cog 2
{10011010}              word    skip            'do_sbb_d 'Cog 2
{10011011}              word    skip            'do_sbb_e 'Cog 2
{10011100}              word    skip            'do_sbb_h 'Cog 2
{10011101}              word    skip            'do_sbb_l 'Cog 2
{10011110}              word    skip            'do_sbb_m 'Cog 2
{10011111}              word    skip            'do_sbb_a 'Cog 2

{10100000}              word    skip            'do_ana_b 'Cog 2
{10100001}              word    skip            'do_ana_c 'Cog 2
{10100010}              word    skip            'do_ana_d 'Cog 2
{10100011}              word    skip            'do_ana_e 'Cog 2
{10100100}              word    skip            'do_ana_h 'Cog 2
{10100101}              word    skip            'do_ana_l 'Cog 2
{10100110}              word    skip            'do_ana_m 'Cog 2
{10100111}              word    skip            'do_ana_a 'Cog 2

{10101000}              word    skip            'do_xra_b 'Cog 2
{10101001}              word    skip            'do_xra_c 'Cog 2
{10101010}              word    skip            'do_xra_d 'Cog 2
{10101011}              word    skip            'do_xra_e 'Cog 2
{10101100}              word    skip            'do_xra_h 'Cog 2
{10101101}              word    skip            'do_xra_l 'Cog 2
{10101110}              word    skip            'do_xra_m 'Cog 2
{10101111}              word    skip            'do_xra_a 'Cog 2

{10110000}              word    skip            'do_ora_b 'Cog 2
{10110001}              word    skip            'do_ora_c 'Cog 2
{10110010}              word    skip            'do_ora_d 'Cog 2
{10110011}              word    skip            'do_ora_e 'Cog 2
{10110100}              word    skip            'do_ora_h 'Cog 2
{10110101}              word    skip            'do_ora_l 'Cog 2
{10110110}              word    skip            'do_ora_m 'Cog 2
{10110111}              word    skip            'do_ora_a 'Cog 2

{10111000}              word    skip            'do_cmp_b 'Cog 2
{10111001}              word    skip            'do_cmp_c 'Cog 2
{10111010}              word    skip            'do_cmp_d 'Cog 2
{10111011}              word    skip            'do_cmp_e 'Cog 2
{10111100}              word    skip            'do_cmp_h 'Cog 2
{10111101}              word    skip            'do_cmp_l 'Cog 2
{10111110}              word    skip            'do_cmp_m 'Cog 2
{10111111}              word    skip            'do_cmp_a 'Cog 2
                        
{11000000}              word    do_rnz 'Cog 3
{11000001}              word    do_pop_b 'Cog 3
{11000010}              word    do_jnz 'Cog 3
{11000011}              word    do_jmp 'Cog 3
{11000100}              word    do_cnz 'Cog 3
{11000101}              word    do_push_b 'Cog 3
{11000110}              word    skip            'do_adi 'Cog 2
{11000111}              word    do_rst_0 'Cog 3
{11001000}              word    do_rz 'Cog 3
{11001001}              word    do_ret 'Cog 3
{11001010}              word    do_jz 'Cog 3
{11001011}              word    skip            'do_undocumented 'Cog 0
{11001100}              word    do_cz 'Cog 3
{11001101}              word    do_call 'Cog 3
{11001110}              word    skip            'do_aci 'Cog 2
{11001111}              word    do_rst_1 'Cog 3
{11010000}              word    do_rnc 'Cog 3
{11010001}              word    do_pop_d 'Cog 3
{11010010}              word    do_jnc 'Cog 3
{11010011}              word    do_out 'Cog 3
{11010100}              word    do_cnc 'Cog 3
{11010101}              word    do_push_d 'Cog 3
{11010110}              word    skip            'do_sui 'Cog 2
{11010111}              word    do_rst_2 'Cog 3
{11011000}              word    do_rc 'Cog 3
{11011001}              word    skip            'do_undocumented 'Cog 0
{11011010}              word    do_jc 'Cog 3
{11011011}              word    do_in 'Cog 3
{11011100}              word    do_cc 'Cog 3
{11011101}              word    skip            'do_undocumented 'Cog 0
{11011110}              word    skip            'do_sbi 'Cog 2
{11011111}              word    do_rst_3 'Cog 3
{11100000}              word    do_rpo 'Cog 3
{11100001}              word    do_pop_h 'Cog 3
{11100010}              word    do_jpo 'Cog 3
{11100011}              word    do_xthl 'Cog 3
{11100100}              word    do_cpo 'Cog 3
{11100101}              word    do_push_h 'Cog 3
{11100110}              word    skip            'do_ani 'Cog 2
{11100111}              word    do_rst_4 'Cog 3
{11101000}              word    do_rpe 'Cog 3
{11101001}              word    do_pchl 'Cog 3
{11101010}              word    do_jpe 'Cog 3
{11101011}              word    do_xchg 'Cog 3
{11101100}              word    do_cpe 'Cog 3
{11101101}              word    skip            'do_undocumented 'Cog 0
{11101110}              word    skip            'do_xri 'Cog 2
{11101111}              word    do_rst_5 'Cog 3
{11110000}              word    do_rp 'Cog 3
{11110001}              word    do_pop_af 'Cog 3
{11110010}              word    do_jp 'Cog 3
{11110011}              word    do_di 'Cog 3
{11110100}              word    do_cp 'Cog 3
{11110101}              word    do_push_af 'Cog 3
{11110110}              word    skip            'do_ori 'Cog 2
{11110111}              word    do_rst_6 'Cog 3
{11111000}              word    do_rm 'Cog 3
{11111001}              word    do_sphl 'Cog 3
{11111010}              word    do_jm 'Cog 3
{11111011}              word    do_ei 'Cog 3
{11111100}              word    do_cm 'Cog 3
{11111101}              word    skip            'do_undocumented 'Cog 0
{11111110}              word    skip            'do_cpi 'Cog 2
{11111111}              word    do_rst_7 'Cog 3
'---------------------------------------------------------------------------------------------------------